@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 '''The main interface to MeCab via natto-py.'''
+import logging
 import os
 import re
 from .api import MeCabError
@@ -10,6 +11,9 @@ from .node import MeCabNode
 from .option_parse import OptionParse
 from .support import string_support, splitter_support
 
+logging.basicConfig()
+logger = logging.getLogger('natto.mecab')
+
 class MeCab(object):
     '''The main interface to the MeCab library, wrapping the MeCab Tagger.
 
@@ -18,8 +22,13 @@ class MeCab(object):
     surface and related features, or for iterating over MeCabNode instances
     which contain detailed information about the morphemes encompassed.
 
-    Use the debug keyword argument when instantiating MeCab to output
-    debugging messages to stderr.
+    Configure logging before instantiating MeCab to see debug messages::
+
+        import logging
+
+        fmt='%(asctime)s : %(levelname)s : %(message)s'
+
+        logging.basicConfig(format=fmt, level=logging.DEBUG)
 
     Example usage::
 
@@ -88,6 +97,10 @@ class MeCab(object):
     _ERROR_INIT = 'Could not initialize MeCab: {}'
     _ERROR_NOTSTR = 'Text should be of type str'
     _ERROR_NULLPTR = 'Could not initialize MeCab {}'
+    _ERROR_MISSING_NL = 'Partial-parsing requires new-line char at end of text'
+    _ERROR_BOUNDARY = 'boundary_constraints must be re or str'
+    _ERROR_FEATURE = 'feature_constraints must be tuple'
+    _ERROR_NODEFORMAT = 'Could not format node with surface {}: {}'
 
     _REPR_FMT = ('<{}.{} model={}, tagger={}, lattice={},'
                  ' libpath="{}", options={}, dicts={}, version={}>')
@@ -105,6 +118,8 @@ class MeCab(object):
 
     _REGEXTYPE = type(re.compile(''))
 
+    _STRIP_WHITESPACE = ' {}'.format(os.linesep)
+
     MECAB_LATTICE_ONE_BEST = 1
     MECAB_LATTICE_NBEST = 2
     MECAB_LATTICE_PARTIAL = 4
@@ -141,7 +156,7 @@ class MeCab(object):
             self.__bytes2str, self.__str2bytes = string_support(env.charset)
 
             # Python 2/3 sentence splitter/tokenizer support
-            self.__split = splitter_support(env.charset)
+            self.__split_pattern, self.__split_features = splitter_support(env.charset)
 
             # Set up dictionary of MeCab options to use
             op = OptionParse(env.charset)
@@ -152,14 +167,17 @@ class MeCab(object):
 
             self.model = self.__mecab.mecab_model_new2(ostr)
             if self.model == self.__ffi.NULL:
+                logger.error(self._ERROR_NULLPTR.format('Model'))
                 raise MeCabError(self._ERROR_NULLPTR.format('Model'))
 
             self.tagger = self.__mecab.mecab_model_new_tagger(self.model)
             if self.tagger == self.__ffi.NULL:
+                logger.error(self._ERROR_NULLPTR.format('Tagger'))
                 raise MeCabError(self._ERROR_NULLPTR.format('Tagger'))
 
             self.lattice = self.__mecab.mecab_model_new_lattice(self.model)
             if self.lattice == self.__ffi.NULL:
+                logger.error(self._ERROR_NULLPTR.format('Lattice'))
                 raise MeCabError(self._ERROR_NULLPTR.format('Lattice'))
 
             n = self.options.get('nbest', 1)
@@ -203,8 +221,10 @@ class MeCab(object):
             self.version = self.__bytes2str(
                 self.__ffi.string(self.__mecab.mecab_version()))
         except EnvironmentError as err:
+            logger.error(self._ERROR_INIT.format(str(err)))
             raise MeCabError(err)
         except ValueError as verr:
+            logger.error(self._ERROR_INIT.format(str(verr)))
             raise MeCabError(self._ERROR_INIT.format(str(verr)))
 
     def __del__(self):
@@ -245,7 +265,7 @@ class MeCab(object):
 
         if self._KW_BOUNDARY in kwargs:
             patt = kwargs.get(self._KW_BOUNDARY, '.')
-            tokens = list(self.__split(patt, text))
+            tokens = list(self.__split_pattern(text, patt))
             text = ''.join([t[0] for t in tokens])
 
             btext = self.__str2bytes(text)
@@ -268,6 +288,23 @@ class MeCab(object):
                     bpos += 1
                 self.__mecab.mecab_lattice_set_boundary_constraint(
                     self.lattice, bpos, self.MECAB_TOKEN_BOUNDARY)
+        elif self._KW_FEATURE in kwargs:
+            features = kwargs.get(self._KW_FEATURE, ())
+            fd = {morph: self.__str2bytes(feat) for morph, feat in features}
+
+            tokens = self.__split_features(text, [e[0] for e in features])
+            text = ''.join([t[0] for t in tokens])
+
+            btext = self.__str2bytes(text)
+            self.__mecab.mecab_lattice_set_sentence(self.lattice, btext)
+
+            bpos = 0
+            for chunk, match in tokens:
+                c = len(self.__str2bytes(chunk))
+                if match == True:
+                    self.__mecab.mecab_lattice_set_feature_constraint(
+                        self.lattice, bpos, bpos+c, fd[chunk])
+                bpos += c
         else:
             btext = self.__str2bytes(text)
             self.__mecab.mecab_lattice_set_sentence(self.lattice, btext)
@@ -281,9 +318,10 @@ class MeCab(object):
 
         if res != self.__ffi.NULL:
             raw = self.__ffi.string(res)
-            return self.__bytes2str(raw).strip()
+            return self.__bytes2str(raw).strip(self._STRIP_WHITESPACE)
         else:
             err = self.__mecab.mecab_lattice_strerror(self.lattice)
+            logger.error(self.__bytes2str(self.__ffi.string(err)))
             raise MeCabError(self.__bytes2str(self.__ffi.string(err)))
 
     def __parse_tonodes(self, text, **kwargs):
@@ -304,7 +342,7 @@ class MeCab(object):
         try:
             if self._KW_BOUNDARY in kwargs:
                 patt = kwargs.get(self._KW_BOUNDARY, '.')
-                tokens = list(self.__split(patt, text))
+                tokens = list(self.__split_pattern(text, patt))
                 text = ''.join([t[0] for t in tokens])
 
                 btext = self.__str2bytes(text)
@@ -327,6 +365,23 @@ class MeCab(object):
                         bpos += 1
                     self.__mecab.mecab_lattice_set_boundary_constraint(
                         self.lattice, bpos, self.MECAB_TOKEN_BOUNDARY)
+            elif self._KW_FEATURE in kwargs:
+                features = kwargs.get(self._KW_FEATURE, ())
+                fd = {morph: self.__str2bytes(feat) for morph, feat in features}
+
+                tokens = self.__split_features(text, [e[0] for e in features])
+                text = ''.join([t[0] for t in tokens])
+
+                btext = self.__str2bytes(text)
+                self.__mecab.mecab_lattice_set_sentence(self.lattice, btext)
+
+                bpos = 0
+                for chunk, match in tokens:
+                    c = len(self.__str2bytes(chunk))
+                    if match:
+                        self.__mecab.mecab_lattice_set_feature_constraint(
+                            self.lattice, bpos, bpos+c, fd[chunk])
+                    bpos += c
             else:
                 btext = self.__str2bytes(text)
                 self.__mecab.mecab_lattice_set_sentence(self.lattice, btext)
@@ -342,22 +397,36 @@ class MeCab(object):
                         if nptr.stat != MeCabNode.BOS_NODE:
                             raws = self.__ffi.string(
                                 nptr.surface[0:nptr.length])
-                            surf = self.__bytes2str(raws).strip()
+                            surf = self.__bytes2str(raws).strip(self._STRIP_WHITESPACE)
 
                             if 'output_format_type' in self.options or \
                                'node_format' in self.options:
                                 sp = self.__mecab.mecab_format_node(
                                     self.tagger, nptr)
-                                rawf = self.__ffi.string(sp)
+                                if sp != self.__ffi.NULL:
+                                    rawf = self.__ffi.string(sp)
+                                else:
+                                    err = self.__mecab.mecab_strerror(
+                                            self.tagger)
+                                    err = self.__bytes2str(
+                                            self.__ffi.string(err))
+                                    msg = self._ERROR_NODEFORMAT.format(
+                                            surf, err)
+                                    raise MeCabError(msg)
                             else:
                                 rawf = self.__ffi.string(nptr.feature)
-                            feat = self.__bytes2str(rawf).strip()
+                            feat = self.__bytes2str(rawf).strip(self._STRIP_WHITESPACE)
 
                             mnode = MeCabNode(nptr, surf, feat)
                             yield mnode
                         nptr = getattr(nptr, 'next')
+        except GeneratorExit:
+            logger.debug('close invoked on generator')
+        except MeCabError:
+            raise
         except:
             err = self.__mecab.mecab_lattice_strerror(self.lattice)
+            logger.error(self.__bytes2str(self.__ffi.string(err)))
             raise MeCabError(self.__bytes2str(self.__ffi.string(err)))
 
     def __repr__(self):
@@ -381,26 +450,38 @@ class MeCab(object):
             or string if False.
         :type as_nodes: bool, defaults to False
         :param boundary_constraints: regular expression for morpheme boundary
-            splitting; if non-None, then boundary constraint parsing will be
-            used.
-        :type boundary_constraints: str
+            splitting; if non-None and feature_constraints is None, then
+            boundary constraint parsing will be used.
+        :type boundary_constraints: str or re
+        :param feature_constraints: tuple containing tuple instances of
+            target morpheme and corresponding feature string in order
+            of precedence; if non-None and boundary_constraints is None,
+            then feature constraint parsing will be used.
+        :type feature_constraints: tuple
         :return: A single string containing the entire MeCab output;
             or a Generator yielding the MeCabNode instances.
         :raises: MeCabError
         '''
         if text is None:
+            logger.error(self._ERROR_EMPTY_STR)
             raise MeCabError(self._ERROR_EMPTY_STR)
         elif not isinstance(text, str):
+            logger.error(self._ERROR_NOTSTR)
             raise MeCabError(self._ERROR_NOTSTR)
+        elif 'partial' in self.options and not text.endswith("\n"):
+            logger.error(self._ERROR_MISSING_NL)
+            raise MeCabError(self._ERROR_MISSING_NL)
 
         if self._KW_BOUNDARY in kwargs:
             val = kwargs[self._KW_BOUNDARY]
             if not isinstance(val, self._REGEXTYPE) and not isinstance(val, str):
-                raise MeCabError('boundary_constraints must be re or str')
+                logger.error(self._ERROR_BOUNDARY)
+                raise MeCabError(self._ERROR_BOUNDARY)
         elif self._KW_FEATURE in kwargs:
             val = kwargs[self._KW_FEATURE]
-            if not isinstance(val, list):
-                raise MeCabError('feature_constraints must be list')
+            if not isinstance(val, tuple):
+                logger.error(self._ERROR_FEATURE)
+                raise MeCabError(self._ERROR_FEATURE)
 
         as_nodes = kwargs.get(self._KW_ASNODES, False)
 
@@ -410,7 +491,7 @@ class MeCab(object):
             return self.__parse_tostr(text, **kwargs)
 
 '''
-Copyright (c) 2015, Brooke M. Fujita.
+Copyright (c) 2020, Brooke M. Fujita.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
