@@ -1,13 +1,16 @@
 # -*- coding: utf-8 -*-
 '''Helper class for parsing MeCab options.'''
 import argparse
-import sys
+import logging
 from .support import string_support
 
+logger = logging.getLogger('natto.option_parse')
+
 class OptionParse(object):
     '''Helper class for transforming arguments into input for mecab_new2.'''
 
-    _SUPPORTED_OPTS = {'-d' : 'dicdir',
+    _SUPPORTED_OPTS = {'-r' : 'rcfile',
+                       '-d' : 'dicdir',
                        '-u' : 'userdic',
                        '-l' : 'lattice_level',
                        '-O' : 'output_format_type',
@@ -74,13 +77,16 @@ class OptionParse(object):
         if type(options) is dict:
             for name in iter(list(self._SUPPORTED_OPTS.values())):
                 if name in options:
-                    if options[name]:
+                    if options[name] or options[name] == '':
                         val = options[name]
                         if isinstance(val, bytes):
                             val = self.__bytes2str(options[name])
                         dopts[name] = val
         else:
             p = MeCabArgumentParser()
+            p.add_argument('-r', '--rcfile',
+                           help='use FILE as a resource file',
+                           action='store', dest='rcfile')
             p.add_argument('-d', '--dicdir',
                            help='set DIR as a system dicdir',
                            action='store', dest='dicdir')
@@ -146,22 +152,23 @@ class OptionParse(object):
                            help='set cost factor (default 700)',
                            action='store', dest='cost_factor', type=int)
 
-            opts = p.parse_args(options.split())
+            opts = p.parse_args([o.replace('\"', '').replace('\'', '') for o in options.split()])
 
             for name in iter(list(self._SUPPORTED_OPTS.values())):
                 if hasattr(opts, name):
                     v = getattr(opts, name)
-                    if v:
+                    if v or v == '':
                         dopts[name] = v
 
         # final checks
         if 'nbest' in dopts \
             and (dopts['nbest'] < 1 or dopts['nbest'] > self._NBEST_MAX):
+            logger.error(self._ERROR_NVALUE)
             raise ValueError(self._ERROR_NVALUE)
 
         # warning for lattice-level deprecation
         if 'lattice_level' in dopts:
-            sys.stderr.write('WARNING: {}\n'.format(self._WARN_LATTICE_LEVEL))
+            logger.warn('WARNING: {}\n'.format(self._WARN_LATTICE_LEVEL))
 
         return dopts
 
@@ -189,7 +196,7 @@ class OptionParse(object):
         return self.__str2bytes(' '.join(opts))
 
 '''
-Copyright (c) 2015, Brooke M. Fujita.
+Copyright (c) 2020, Brooke M. Fujita.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
