@@ -54,11 +54,11 @@ try:
         line = line.decode(os.getenv(MeCab.MECAB_CHARSET))
     if not line.startswith(exp):
         raise EnvironmentError('Please check your mecab installation')
-except StandardError as err:
+except Exception as err:
     raise EnvironmentError(err)
 
 '''
-Copyright (c) 2015, Brooke M. Fujita.
+Copyright (c) 2020, Brooke M. Fujita.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
