@@ -9,8 +9,12 @@ import unittest
 import natto.api as api
 import natto.environment as env
 import natto.mecab as mecab
+import natto.support as support
+from os import path
+from string import Template
 from subprocess import Popen, PIPE
 from tests import Test23Support
+from yaml import FullLoader
 
 class TestMecab(unittest.TestCase, Test23Support):
     '''Tests the behavior of the natto.mecab.MeCab class.
@@ -25,35 +29,56 @@ class TestMecab(unittest.TestCase, Test23Support):
         else:
             self.textfile = os.path.join(cwd, 'tests', 'test_utf8.txt')
 
-        self.yamlfile = os.path.join(cwd, 'tests', 'test_utf8.yml')
+        yamlfile = os.path.join(cwd, 'tests', 'test_utf8.yml')
         self.env = env.MeCabEnv()
 
+        self.b2s, self.s2b = support.string_support(self.env.charset)
+
+        self.testrc = os.path.join(cwd, 'tests', 'testmecabrc')
+
         with codecs.open(self.textfile, 'r') as f:
-            self.text = f.readlines()[0].strip()
+            self.text = f.readlines()[0].strip(' {}'.format(os.linesep))
+
+        with codecs.open(yamlfile, 'r', encoding='utf-8') as f:
+            self.yaml = yaml.load(f, Loader=FullLoader)
+
+        cmd = ['mecab', '-P']
+        mout = Popen(cmd, stdout=PIPE).communicate()
+        res = self.b2s(mout[0])
+        m = re.search('(?<=dicdir:\s).*', res)
+        ipadic = path.abspath(m.group(0).strip(' {}'.format(os.linesep)))
+        with open(path.join(os.getcwd(), 'tests', 'mecabrc.tmp'), 'r') as fin:
+            tmpl = Template(fin.read())
+
+            tmpl = tmpl.substitute({'ipadic': ipadic})
+
+            with open(self.testrc, 'w') as fout:
+                fout.write(tmpl)
 
-        with codecs.open(self.yamlfile, 'r', encoding='utf-8') as f:
-            self.yaml = yaml.load(f)
 
     def tearDown(self):
         self.textfile = None
         self.text = None
         self.yaml = None
         self.env = None
+        self.testrc = None
 
     def _mecab_parse(self, options):
         cmd = ['mecab']
-        if sys.platform == 'win32':
+
+        if type(options) is str:
             if len(options) > 0:
                 cmd.append(options)
-            cmd.append(self.textfile)
+        elif type(options) is list:
+            cmd.extend(options)
+        cmd.append(self.textfile)
+
+        if sys.platform == 'win32':
             mout = Popen(cmd, stdout=PIPE, shell=True).communicate()
         else:
-            if len(options) > 0:
-                cmd.append(options)
-            cmd.append(self.textfile)
             mout = Popen(cmd, stdout=PIPE).communicate()
 
-        res = mout[0].strip()
+        res = mout[0].strip(' {}'.format(os.linesep).encode())
         return res
 
     def _23support_prep(self, morphs):
@@ -94,10 +119,32 @@ class TestMecab(unittest.TestCase, Test23Support):
     # ------------------------------------------------------------------------
     def test_parse_args(self):
         '''Test invocation of parse with bad arguments.'''
+        # None text
+        with mecab.MeCab() as nm:
+            with self.assertRaises(api.MeCabError):
+                nm.parse(None)
+
+        # text must be str
+        with mecab.MeCab() as nm:
+            with self.assertRaises(api.MeCabError):
+                nm.parse(99)
+
+        # boundary_constraints must be re or str
         with mecab.MeCab() as nm:
             with self.assertRaises(api.MeCabError):
                 nm.parse('foo', boundary_constraints=99.99)
 
+        # feature_constraints must be tuple
+        with mecab.MeCab() as nm:
+            with self.assertRaises(api.MeCabError):
+                nm.parse('foo', feature_constraints=[])
+
+        # -p / --partial, text must end with \n
+        with mecab.MeCab('--partial') as nm:
+            with self.assertRaises(api.MeCabError):
+                nm.parse('foo')
+
+
     def test_parse_unicodeRstr(self):
         '''Test parse: unicode input (Python 2) and bytes input (Python 3).'''
         s = '日本語だよ、これが。'
@@ -110,17 +157,11 @@ class TestMecab(unittest.TestCase, Test23Support):
             with self.assertRaises(api.MeCabError):
                 nm.parse(b)
 
-    def test_null_text_error(self):
-        '''Test invocation of parse with null argument.'''
-        with mecab.MeCab() as nm:
-            with self.assertRaises(api.MeCabError):
-                nm.parse(None)
-
     # ------------------------------------------------------------------------
     def test_parse_tostr_default(self):
         '''Test simple default parsing.'''
         with mecab.MeCab() as nm:
-            expected = nm.parse(self.text).strip()
+            expected = nm.parse(self.text).strip(' {}'.format(os.linesep))
             expected = expected.replace('\n', os.linesep)                 # ???
 
             actual = self._2bytes(self._mecab_parse(''))
@@ -144,7 +185,7 @@ class TestMecab(unittest.TestCase, Test23Support):
 
                 self.assertEqual(expected, actual)
 
-   # ------------------------------------------------------------------------
+    # ------------------------------------------------------------------------
     def test_parse_tonode_default(self):
         '''Test node parsing, skipping over any BOS or EOS nodes.'''
         formats = ['', '-N2']
@@ -156,13 +197,38 @@ class TestMecab(unittest.TestCase, Test23Support):
                 actual = self._2bytes(self._mecab_parse(argf))
                 actual = [e for e in actual.split(os.linesep) if e != 'EOS']
 
-                for i, e in enumerate(actual):
-                    s, f = actual[i].split()
+                for i in range(len(actual)):
+                    s, f = actual[i].split('\t')
                     self.assertEqual(expected[i].surface, s)
                     self.assertEqual(expected[i].feature, f)
 
-   # ------------------------------------------------------------------------
-    def test_bcparse_tostr(self):
+    def test_parse_tonode_outputformat_errors(self):
+        '''Test node parsing with output formatting errors:
+           1. unknown node has no pronunciation value
+           2. format missing leading [
+           3. format missing ending ]
+        '''
+        s = '私はブルザエモンです。'
+        formats = ['-F%f[8]', '-F%f1]', '-F%f[1']
+        for argf in formats:
+            with mecab.MeCab(argf) as nm:
+                with self.assertRaises(api.MeCabError):
+                    list(nm.parse(s, as_nodes=True))
+
+    # ------------------------------------------------------------------------
+    def test_parse_tostr_partial(self):
+        '''Test -p / --partial parsing to string.'''
+        with mecab.MeCab('-p') as nm:
+            yml = self.yaml.get('text10')
+            txt = self._u2str(yml.get('text'))
+            actual = nm.parse(txt).split('\n')
+            expected = [self._u2str(e) for e in yml.get('expected').get('str').split(',')]
+
+            for i in range(len(actual)):
+                self.assertTrue(actual[i].startswith(expected[i]))
+
+    # ------------------------------------------------------------------------
+    def test_parse_tostr_boundary(self):
         '''Test boundary constraint parsing to string (output format does NOT apply).'''
         with mecab.MeCab() as nm:
             # simple pattern
@@ -174,7 +240,7 @@ class TestMecab(unittest.TestCase, Test23Support):
             actual = nm.parse(txt1, boundary_constraints=pat1)
             lines = actual.split(os.linesep)
 
-            for i, e in enumerate(lines):
+            for i in range(len(lines)):
                 self.assertTrue(lines[i].startswith(expected[i]))
 
             # slightly more complex pattern
@@ -186,7 +252,7 @@ class TestMecab(unittest.TestCase, Test23Support):
             actual = nm.parse(txt2, boundary_constraints=pat2)
             lines = actual.split(os.linesep)
 
-            for i, e in enumerate(lines):
+            for i in range(len(lines)):
                 self.assertTrue(lines[i].startswith(expected[i]))
 
             # complex pattern requiring RegExp compiled with re.U flag
@@ -198,19 +264,7 @@ class TestMecab(unittest.TestCase, Test23Support):
             actual = nm.parse(txt3, boundary_constraints=re.compile(pat3, re.U))
             lines = actual.split(os.linesep)
 
-            for i, e in enumerate(lines):
-                self.assertTrue(lines[i].startswith(expected[i]))
-
-            # text includes trailing whitespace char in token
-            yml9 = self.yaml.get('text9')
-            txt9 = self._u2str(yml9.get('text'))
-            pat9 = self._u2str(yml9.get('pattern'))
-            expected = [self._u2str(e) for e in yml9.get('expected')]
-
-            actual = nm.parse(txt9, boundary_constraints=pat9)
-            lines = actual.split(os.linesep)
-
-            for i, e in enumerate(lines):
+            for i in range(len(lines)):
                 self.assertTrue(lines[i].startswith(expected[i]))
 
         with mecab.MeCab('-N2') as nm:
@@ -223,7 +277,7 @@ class TestMecab(unittest.TestCase, Test23Support):
             actual = nm.parse(txt, boundary_constraints=pat)
             lines = actual.splitlines()
 
-            for i, e in enumerate(lines):
+            for i in range(len(lines)):
                 self.assertTrue(lines[i].endswith(expected[i]))
 
         # with theta option
@@ -238,11 +292,11 @@ class TestMecab(unittest.TestCase, Test23Support):
                 actual = nm.parse(txt1, boundary_constraints=pat1)
                 lines = actual.split(os.linesep)
 
-                for i, e in enumerate(lines):
+                for i in range(len(lines)):
                     self.assertTrue(lines[i].startswith(expected[i]))
 
-   # ------------------------------------------------------------------------
-    def test_bcparse_tonodes(self):
+    # ------------------------------------------------------------------------
+    def test_parse_tonodes_boundary(self):
         '''Test boundary constraint parsing as nodes (output format does NOT apply).'''
         with mecab.MeCab() as nm:
             # simple node-parsing, no N-Best or output formatting
@@ -267,17 +321,6 @@ class TestMecab(unittest.TestCase, Test23Support):
                 if not node.is_eos():
                     self.assertEqual(node.surface, expected[i])
 
-            # text includes trailing whitespace char in token
-            yml9 = self.yaml.get('text9')
-            txt9 = self._u2str(yml9.get('text'))
-            pat9 = self._u2str(yml9.get('pattern'))
-            expected = [self._u2str(e) for e in yml9.get('expected')]
-
-            gen = nm.parse(txt9, boundary_constraints=pat9, as_nodes=True)
-            for i, node in enumerate(gen):
-                if not node.is_eos():
-                    self.assertEqual(node.surface, expected[i])
-
         with mecab.MeCab(r'-F%m\s%s') as nm:
             # with output formatting
             yml1 = self.yaml.get('text5')
@@ -317,10 +360,37 @@ class TestMecab(unittest.TestCase, Test23Support):
                     if not node.is_eos():
                         self.assertEqual(node.surface, expected[i])
 
+    # ------------------------------------------------------------------------
+    def test_parse_tostr_feature(self):
+        '''Test feature constraint parsing to string (output format does NOT apply).'''
+        with mecab.MeCab(r'-F%m,%f[0],%s\n') as nm:
+            yml = self.yaml.get('text11')
+            txt = self._u2str(yml.get('text'))
+            feat = (tuple(self._u2str(yml.get('feature')).split(',')) ,)
+            expected = [self._u2str(e) for e in yml.get('expected')]
+
+            actual = nm.parse(txt, feature_constraints=feat).split('\n')
+
+            for i in range(len(actual)):
+                self.assertEqual(actual[i], expected[i])
+
+    # ------------------------------------------------------------------------
+    def test_parse_override_node_format(self):
+        '''Test node-format override when default is defined in rcfile'''
+        with mecab.MeCab(r'-r {} -O "" -F%m!\n'.format(self.testrc)) as nm:
+            expected = nm.parse(self.text, as_nodes=True)
+            expected = [e.feature for e in expected if e.stat == 0]
+
+            argf = ['-r', self.testrc, '-O', '', '-F%m!\\n']
+            actual = self._2bytes(self._mecab_parse(argf))
+            actual = [e for e in actual.split(os.linesep) if not e.startswith('EOS')]
+
+            for i,e in enumerate(actual):
+                self.assertEqual(e, expected[i])
 
 
 '''
-Copyright (c) 2015, Brooke M. Fujita.
+Copyright (c) 2020, Brooke M. Fujita.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
